@@ -6,8 +6,8 @@
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
 Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This part of the software is released under ONE of the following licenses:
-GPL OR Genivia's license for commercial use.
+This part of the software is released under one of the following licenses:
+GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -43,8 +43,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "error2.h"
 
 #ifndef VERSION
-# define VERSION "2.8.12" /* Current version */
-# define GSOAP_VERSION 20812
+# define VERSION "2.8.16" /* Current version */
+# define GSOAP_VERSION 20816
 #endif
 
 #ifdef WIN32
@@ -252,7 +252,7 @@ typedef	struct IDinfo {
 
 typedef	struct Entry {
 	Symbol	*sym;
-	Symbol	*tag;
+	char	*tag;
 	IDinfo	info;
 	Level	level;
 	int	lineno;
@@ -296,7 +296,7 @@ typedef	struct Node {
 #define RESPONSE_ENCODING       0x0400
 #define STYLE                   0x0800
 #define FAULT                   0x1000
-#define TRANSPORT               0x2000
+#define PROTOCOL                0x2000
 
 typedef struct Data
 {	struct Data *next;
@@ -328,6 +328,7 @@ typedef struct Service
 	char *WSDL;
 	char *style;
 	char *encoding;
+	char *protocol;
 	int xsi_type;
 	char *elementForm;
 	char *attributeForm;
