@@ -94,8 +94,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 	https://127.0.0.1:8081
 	and type userid 'admin' and passwd 'guest' to gain access
 	Open the location:
-	https://127.0.0.1:8081/calc.html
-	and enter an expression
+	https://127.0.0.1:8081/calcform1.html
+	and enter an expression to calculate
 	Open the locations:
 	https://127.0.0.1:8081/test.html
 	https://127.0.0.1:8081/webserver.wsdl
@@ -128,7 +128,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 */
 
 #include "soapH.h"
-#include "webserver.nsmap"
+#include "webserver.nsmap"	/* namespaces updated 4/4/13 */
 #include "options.h"
 #include "httpget.h"
 #include "httpform.h"
@@ -179,11 +179,11 @@ static const struct option default_options[] =
   { "k.keepalive", NULL, },
   { "i.iterative", NULL, },
   { "v.verbose", NULL, },
-  { "o.pool", "threads", 6, "none"},
-  { "t.ioTimeout", "seconds", 6, "5"},
-  { "s.serverTimeout", "seconds", 6, "3600"},
-  { "d.cookieDomain", "host", 20, "127.0.0.1"},
-  { "p.cookiePath", "path", 20, "/"},
+  { "o.pool", "threads", 6, (char*)"none"},
+  { "t.ioTimeout", "seconds", 6, (char*)"5"},
+  { "s.serverTimeout", "seconds", 6, (char*)"3600"},
+  { "d.cookieDomain", "host", 20, (char*)"127.0.0.1"},
+  { "p.cookiePath", "path", 20, (char*)"/"},
   { "l.logging", "none inbound outbound both", },
   { "", "port", },		/* takes the rest of command line args */
   { NULL },			/* must be NULL terminated */
@@ -536,6 +536,7 @@ void *process_request(void *soap)
   {
     soap_print_fault(tsoap, stderr);
     fprintf(stderr, "SSL request failed, continue with next call...\n");
+    soap_destroy(tsoap);
     soap_end(tsoap);
     soap_done(tsoap);
     free(tsoap);
@@ -551,7 +552,7 @@ void *process_request(void *soap)
   else if (options[OPTION_v].selected)
     fprintf(stderr, "Thread %d completed\n", (int)(long)tsoap->user);
 
-  soap_destroy((struct soap*)soap);
+  soap_destroy(tsoap);
   soap_end(tsoap);
   soap_done(tsoap);
   free(soap);
@@ -583,6 +584,7 @@ void *process_queue(void *soap)
     {
       soap_print_fault(tsoap, stderr);
       fprintf(stderr, "SSL request failed, continue with next call...\n");
+      soap_destroy(tsoap);
       soap_end(tsoap);
       continue;
     }
@@ -743,7 +745,7 @@ int http_get_handler(struct soap *soap)
 #endif
   /* Use soap->path (from request URL) to determine request: */
   if (options[OPTION_v].selected)
-    fprintf(stderr, "HTTP GET Request: %s\n", soap->endpoint);
+    fprintf(stderr, "HTTP GET Request '%s' to host '%s' path '%s'\n", soap->endpoint, soap->host, soap->path);
   /* Note: soap->path always starts with '/' */
   if (strchr(soap->path + 1, '/') || strchr(soap->path + 1, '\\'))	/* we don't like snooping in dirs */
     return 403; /* HTTP forbidden */
@@ -785,7 +787,7 @@ int check_authentication(struct soap *soap)
 #ifdef HTTPDA_H
   else if (soap->authrealm && soap->userid)
   { if (!strcmp(soap->authrealm, AUTH_REALM) && !strcmp(soap->userid, AUTH_USERID))
-      if (!http_da_verify_get(soap, AUTH_PASSWD))
+      if (!http_da_verify_get(soap, (char*)AUTH_PASSWD))
         return SOAP_OK;
   }
 #endif
