@@ -5,8 +5,8 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2011, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -37,13 +37,12 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 #include "stdsoap2.h"
 
+#define WSDL2H_VERSION "2.8.16"
+
 #ifdef WIN32
 # pragma warning(disable : 4996)
 #endif
 
-#undef VERSION
-#define VERSION "2.8.7"
-
 #include <utility>
 #include <iterator>
 #include <vector>
@@ -58,6 +57,14 @@ struct ltstr
   }
 }; 
 
+struct eqstr
+{ const char *s;
+  eqstr(const char *s) : s(s) { }
+  bool operator()(const char *t) const
+  { return strcmp(s, t) == 0;
+  }
+}; 
+
 typedef set<const char*, ltstr> SetOfString;
 
 typedef pair<const char*, const char*> Pair;
@@ -65,9 +72,9 @@ typedef pair<const char*, const char*> Pair;
 struct ltpair
 { bool operator()(Pair s1, Pair s2) const
   { int cmp = strcmp(s1.first, s2.first);
-    if (cmp)
-      return cmp < 0;
-    return strcmp(s1.second, s2.second) < 0;
+    if (cmp == 0)
+      cmp = strcmp(s1.second, s2.second);
+    return cmp < 0;
   }
 };
 
@@ -81,6 +88,7 @@ typedef vector<const char*> VectorOfString;
 
 extern int _flag,
            aflag,
+           bflag,
 	   cflag,
 	   dflag,
 	   eflag,
@@ -92,6 +100,7 @@ extern int _flag,
 	   mflag,
 	   pflag,
 	   Pflag,
+	   Rflag,
 	   sflag,
 	   uflag,
 	   vflag,
