@@ -5,8 +5,8 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2011, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -313,7 +313,15 @@ int xs__schema::read(const char *cwd, const char *loc)
   if (vflag)
     fprintf(stderr, "\nOpening schema '%s' from '%s'\n", loc?loc:"", cwd?cwd:"");
   if (loc)
-  {
+  { if (soap->recvfd > 2)
+    { soap_end_recv(soap);
+      close(soap->recvfd);
+      soap->recvfd = -1;
+    }
+    else if (soap_valid_socket(soap->socket))
+    { soap_end_recv(soap);
+      soap_closesock(soap);
+    }
 #ifdef WITH_OPENSSL
     if (!strncmp(loc, "http://", 7) || !strncmp(loc, "https://", 8))
 #else
@@ -460,6 +468,10 @@ const SetOfString& xs__schema::builtinAttributes() const
 { return builtinAttributeSet;
 }
 
+bool xs__schema::empty() const
+{ return include.empty() && redefine.empty() && import.empty() && attribute.empty() && element.empty() && group.empty() && attributeGroup.empty() && simpleType.empty() && complexType.empty();
+}
+
 xs__include::xs__include()
 { schemaLocation = NULL;
   schemaRef = NULL;
@@ -763,12 +775,13 @@ int xs__element::traverse(xs__schema &schema)
   elementRef = NULL;
   if (token)
   { for (vector<xs__element>::iterator i = schema.element.begin(); i != schema.element.end(); ++i)
-      if (!strcmp((*i).name, token))
+    { if ((*i).name && !strcmp((*i).name, token))
       { elementRef = &(*i);
         if (vflag)
           cerr << "    Found element '" << (name?name:"") << "' ref '" << (token?token:"") << "'" << endl;
         break;
       }
+    }
   }
   if (!elementRef)
   { for (vector<xs__import>::const_iterator i = schema.import.begin(); i != schema.import.end(); ++i)
@@ -777,7 +790,7 @@ int xs__element::traverse(xs__schema &schema)
       { token = qname_token(ref, s->targetNamespace);
         if (token)
         { for (vector<xs__element>::iterator j = s->element.begin(); j != s->element.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { elementRef = &(*j);
               if (vflag)
                 cerr << "    Found element '" << (name?name:"") << "' ref '" << (token?token:"") << "'" << endl;
@@ -799,7 +812,7 @@ int xs__element::traverse(xs__schema &schema)
     simpleTypeRef = NULL;
     if (token)
     { for (vector<xs__simpleType>::iterator i = schema.simpleType.begin(); i != schema.simpleType.end(); ++i)
-        if (!strcmp((*i).name, token))
+        if ((*i).name && !strcmp((*i).name, token))
         { simpleTypeRef = &(*i);
           if (vflag)
             cerr << "    Found element '" << (name?name:"") << "' simpleType '" << (token?token:"") << "'" << endl;
@@ -813,7 +826,7 @@ int xs__element::traverse(xs__schema &schema)
         { token = qname_token(type, s->targetNamespace);
           if (token)
           { for (vector<xs__simpleType>::iterator j = s->simpleType.begin(); j != s->simpleType.end(); ++j)
-            { if (!strcmp((*j).name, token))
+            { if ((*j).name && !strcmp((*j).name, token))
               { simpleTypeRef = &(*j);
                 if (vflag)
                   cerr << "    Found element '" << (name?name:"") << "' simpleType '" << (token?token:"") << "'" << endl;
@@ -836,7 +849,7 @@ int xs__element::traverse(xs__schema &schema)
     complexTypeRef = NULL;
     if (token)
     { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-        if (!strcmp((*i).name, token))
+        if ((*i).name && !strcmp((*i).name, token))
         { complexTypeRef = &(*i);
           if (vflag)
             cerr << "    Found element '" << (name?name:"") << "' complexType '" << (token?token:"") << "'" << endl;
@@ -850,7 +863,7 @@ int xs__element::traverse(xs__schema &schema)
         { token = qname_token(type, s->targetNamespace);
           if (token)
           { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-            { if (!strcmp((*j).name, token))
+            { if ((*j).name && !strcmp((*j).name, token))
               { complexTypeRef = &(*j);
                 if (vflag)
                   cerr << "    Found element '" << (name?name:"") << "' complexType '" << (token?token:"") << "'" << endl;
@@ -1156,7 +1169,7 @@ int xs__extension::traverse(xs__schema &schema)
   complexTypeRef = NULL;
   if (token)
   { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-      if (!strcmp((*i).name, token))
+      if ((*i).name && !strcmp((*i).name, token))
       { complexTypeRef = &(*i);
         if (vflag)
           cerr << "    Found extension base type '" << (token?token:"") << "'" << endl;
@@ -1170,7 +1183,7 @@ int xs__extension::traverse(xs__schema &schema)
       { token = qname_token(base, s->targetNamespace);
         if (token)
         { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { complexTypeRef = &(*j);
               if (vflag)
                 cerr << "    Found extension base type '" << (token?token:"") << "'" << endl;
@@ -1220,6 +1233,8 @@ xs__restriction::xs__restriction()
 int xs__restriction::traverse(xs__schema &schema)
 { if (vflag)
     cerr << "   Analyzing schema restriction '" << (base?base:"") << "'" << endl;
+  if (simpleType)
+    simpleType->traverse(schema);
   if (attributeGroup)
     attributeGroup->traverse(schema);
   if (group)
@@ -1273,7 +1288,7 @@ int xs__restriction::traverse(xs__schema &schema)
   complexTypeRef = NULL;
   if (token)
   { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-      if (!strcmp((*i).name, token))
+      if ((*i).name && !strcmp((*i).name, token))
       { complexTypeRef = &(*i);
         if (vflag)
           cerr << "    Found restriction base type '" << (token?token:"") << "'" << endl;
@@ -1287,7 +1302,7 @@ int xs__restriction::traverse(xs__schema &schema)
       { token = qname_token(base, s->targetNamespace);
         if (token)
         { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { complexTypeRef = &(*j);
               if (vflag)
                 cerr << "    Found restriction base type '" << (token?token:"") << "'" << endl;
@@ -1307,7 +1322,7 @@ int xs__restriction::traverse(xs__schema &schema)
       else if (!Wflag)
 	cerr << "Warning: could not find restriction base type '" << base << "' in schema '" << (schema.targetNamespace?schema.targetNamespace:"") << "'" << endl;
     }
-    else
+    else if (!simpleType)
       cerr << "Restriction has no base" << endl;
   }
   return SOAP_OK;
