@@ -6,7 +6,7 @@
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
 Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -54,7 +54,7 @@ class Message
     vector<soap__header> header;
     vector<wsoap__header> wheader;
     mime__multipartRelated *multipartRelated;	// MIME
-    mime__content *content;			// MIME
+    mime__content *content;			// REST/MIME
     const char *layout;				// DIME
     const char *documentation;
     const char *ext_documentation;
