@@ -5,8 +5,8 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+Copyright (C) 2000-2013, Robert van Engelen, Genivia Inc. All Rights Reserved.
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -70,6 +70,7 @@ int _flag = 0,
     mflag = 0,
     pflag = 0,
     Pflag = 0,
+    Rflag = 0,
     sflag = 0,
     uflag = 0,
     vflag = 0,
@@ -118,17 +119,17 @@ const char serviceformat[]             = "//gsoap %-4s service %s:\t%s %s\n";
 const char paraformat[]                = "    %-35s%s%s%s";
 const char anonformat[]                = "    %-35s%s_%s%s";
 
-const char copyrightnotice[] = "\n**  The gSOAP WSDL/Schema processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2012 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool is released under one of the following two licenses:\n**  GPL or the commercial license by Genivia Inc. Use option -l for details.\n\n";
+const char copyrightnotice[] = "\n**  The gSOAP WSDL/Schema processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2013 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool is released under one of the following licenses:\n**  GPL or the commercial license by Genivia Inc. Use option -l for details.\n\n";
 
 const char licensenotice[]   = "\
 --------------------------------------------------------------------------------\n\
 gSOAP XML Web services tools\n\
-Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
+Copyright (C) 2000-2013, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
 \n\
-This software is released under one of the following two licenses:\n\
-1) GPL or 2) Genivia's license for commercial use.\n\
+This software is released under one of the following licenses:\n\
+GPL or Genivia's license for commercial use.\n\
 --------------------------------------------------------------------------------\n\
-1) GPL license.\n\
+GPL license.\n\
 \n\
 This program is free software; you can redistribute it and/or modify it under\n\
 the terms of the GNU General Public License as published by the Free Software\n\
@@ -149,7 +150,7 @@ engelen@genivia.com / engelen@acm.org\n\
 This program is released under the GPL with the additional exemption that\n\
 compiling, linking, and/or using OpenSSL is allowed.\n\
 --------------------------------------------------------------------------------\n\
-2) A commercial-use license is available from Genivia, Inc., contact@genivia.com\n\
+A commercial-use license is available from Genivia, Inc., contact@genivia.com\n\
 --------------------------------------------------------------------------------\n";
 
 int main(int argc, char **argv)
@@ -341,6 +342,9 @@ static void options(int argc, char **argv)
 	      }
 	    }
 	    break;
+	  case 'R':
+	    Rflag = 1;
+	    break;
 	  case 's':
 	    sflag = 1;
 	    break;
@@ -384,10 +388,10 @@ static void options(int argc, char **argv)
 	    break;
           case '?':
           case 'h':
-            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c] [-d] [-e] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-l] [-m] [-N name] [-n name] [-P|-p] [-q name] [-r proxyhost[:port[:uid:pwd]]] [-s] [-t typemapfile] [-u] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
+            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c] [-d] [-e] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-l] [-m] [-N name] [-n name] [-P|-p] [-q name] [-R] [-r proxyhost[:port[:uid:pwd]]] [-s] [-t typemapfile] [-u] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
             fprintf(stderr, "\
 -a      generate indexed struct names for local elements with anonymous types\n\
--b	bi-directional operations (duplex ops) to serve one-way response messages\n\
+-b	bi-directional operations (duplex ops) added to serve one-way responses\n\
 -c      generate C source code\n\
 -d      use DOM to populate xs:any, xs:anyType, and xs:anyAttribute\n\
 -e      don't qualify enum names\n\
@@ -400,12 +404,13 @@ static void options(int argc, char **argv)
 -k      don't generate SOAP_ENV__Header mustUnderstand qualifiers\n\
 -l      display license information\n\
 -m      use xsd.h module to import primitive types\n\
--Nname  use name for service namespace prefixes and multiple bindings\n\
+-Nname  use name for service prefixes to produce a service for each binding\n\
 -nname  use name as the base namespace prefix instead of 'ns'\n\
 -ofile  output to file\n\
 -P      don't create polymorphic types inherited from xsd__anyType\n\
 -p      create polymorphic types inherited from base xsd__anyType\n\
 -qname  use name for the C++ namespace of all declarations\n\
+-R      generate REST operations for REST bindings in the WSDL\n\
 -rhost[:port[:uid:pwd]]\n\
         connect via proxy host, port, and proxy credentials\n\
 -s      don't generate STL code (no std::string and no std::vector)\n\
@@ -420,6 +425,7 @@ static void options(int argc, char **argv)
 -z2     compatibility with 2.7.7 to 2.7.15: qualify element/attribute references\n\
 -z3     compatibility with 2.7.16 to 2.8.7: qualify element/attribute references\n\
 -z4     compatibility up to 2.8.11: don't generate union structs in std::vector\n\
+-z5     compatibility up to 2.8.15\n\
 -_      don't generate _USCORE (replace with UNICODE _x005f)\n\
 infile.wsdl infile.xsd http://www... list of input sources (if none: use stdin)\n\
 \n");
